@@ -14,11 +14,11 @@ Copyright 2022 (C)
 Create structure for OME-NGFF zarr array
 """
 import os
-from glob import glob
 from pathlib import Path
 from typing import Any
 from typing import Dict
 from typing import List
+from typing import Optional
 from typing import Sequence
 
 import pandas as pd
@@ -29,10 +29,12 @@ import fractal_tasks_core
 from fractal_tasks_core.lib_channels import check_well_channel_labels
 from fractal_tasks_core.lib_channels import define_omero_channels
 from fractal_tasks_core.lib_channels import validate_allowed_channel_input
+from fractal_tasks_core.lib_glob import glob_with_multiple_patterns
 from fractal_tasks_core.lib_metadata_parsing import parse_yokogawa_metadata
 from fractal_tasks_core.lib_parse_filename_metadata import parse_filename
 from fractal_tasks_core.lib_regions_of_interest import prepare_FOV_ROI_table
 from fractal_tasks_core.lib_regions_of_interest import prepare_well_ROI_table
+from fractal_tasks_core.lib_remove_FOV_overlaps import remove_FOV_overlaps
 
 
 __OME_NGFF_VERSION__ = fractal_tasks_core.__OME_NGFF_VERSION__
@@ -44,9 +46,11 @@ logger = logging.getLogger(__name__)
 
 def create_ome_zarr(
     *,
-    input_paths: Sequence[Path],
-    output_path: Path,
+    input_paths: Sequence[str],
+    output_path: str,
     metadata: Dict[str, Any],
+    image_extension: str = "tif",
+    image_glob_patterns: Optional[list[str]] = None,
     allowed_channels: Sequence[Dict[str, Any]],
     num_levels: int = 2,
     coarsening_xy: int = 2,
@@ -69,26 +73,33 @@ def create_ome_zarr(
     :param input_paths: TBD (common to all tasks)
     :param output_path: TBD (common to all tasks)
     :param metadata: TBD (common to all tasks)
-    :param allowed_channels: TBD
-    :param num_levels: number of resolution-pyramid levels
-    :param coarsening_xy: linear coarsening factor between subsequent levels
-    :param metadata_table: TBD
+    :param image_extension: Filename extension of images (e.g. `"tif"` or
+                            `"png"`)
+    :param image_glob_patterns: If specified, only parse images with filenames
+                                that match with all these patterns. Patterns
+                                must be defined as in
+                                https://docs.python.org/3/library/fnmatch.html,
+                                e.g. `image_glob_pattern=["*_B03_*"]`.
+    :param num_levels: Number of resolution-pyramid levels
+    :param coarsening_xy: Linear coarsening factor between subsequent levels
+    :param allowed_channels: A list of channel dictionaries, where each channel
+                             must include the ``wavelength_id`` key and where
+                             the corresponding values should be unique across
+                             channels.
+    :param metadata_table: If equal to ``"mrf_mlf"``, parse Yokogawa metadata
+                           from mrf/mlf files in the input_path folder; else,
+                           the full path to a csv file containing
+                           the parsed metadata table.
     """
 
     # Preliminary checks on metadata_table
-    if metadata_table != "mrf_mlf" and not isinstance(
-        metadata_table, pd.core.frame.DataFrame
-    ):
-        raise Exception(
-            "ERROR: metadata_table must be a known string or a "
-            "pandas DataFrame}"
-        )
-    if metadata_table != "mrf_mlf":
-        raise NotImplementedError(
-            "We currently only support "
-            'metadata_table="mrf_mlf", '
-            f"and not {metadata_table}"
+    if metadata_table != "mrf_mlf" and not metadata_table.endswith(".csv"):
+        raise ValueError(
+            "metadata_table must be a known string or a "
+            "csv file containing a pandas dataframe"
         )
+    if metadata_table.endswith(".csv") and not os.path.isfile(metadata_table):
+        raise FileNotFoundError(f"Missing file: {metadata_table=}")
 
     # Identify all plates and all channels, across all input folders
     plates = []
@@ -99,18 +110,23 @@ def create_ome_zarr(
     # Preliminary checks on allowed_channels argument
     validate_allowed_channel_input(allowed_channels)
 
-    # FIXME
-    # find a smart way to remove it
-    ext_glob_pattern = input_paths[0].name
+    for in_path_str in input_paths:
+        in_path = Path(in_path_str)
 
-    for in_path in input_paths:
-        input_filename_iter = in_path.parent.glob(in_path.name)
+        # Glob image filenames
+        patterns = [f"*.{image_extension}"]
+        if image_glob_patterns:
+            patterns.extend(image_glob_patterns)
+        input_filenames = glob_with_multiple_patterns(
+            folder=in_path_str,
+            patterns=patterns,
+        )
 
         tmp_wavelength_ids = []
         tmp_plates = []
-        for fn in input_filename_iter:
+        for fn in input_filenames:
             try:
-                filename_metadata = parse_filename(fn.name)
+                filename_metadata = parse_filename(Path(fn).name)
                 plate_prefix = filename_metadata["plate_prefix"]
                 plate = filename_metadata["plate"]
                 if plate not in dict_plate_prefixes.keys():
@@ -121,22 +137,24 @@ def create_ome_zarr(
                 tmp_wavelength_ids.append(f"A{A}_C{C}")
             except ValueError as e:
                 logger.warning(
-                    f'Skipping "{fn.name}". Original error: ' + str(e)
+                    f'Skipping "{Path(fn).name}". Original error: ' + str(e)
                 )
         tmp_plates = sorted(list(set(tmp_plates)))
         tmp_wavelength_ids = sorted(list(set(tmp_wavelength_ids)))
 
         info = (
-            f"Listing all plates/channels from {in_path.as_posix()}\n"
+            "Listing plates/channels:\n"
+            f"Folder:   {in_path_str}\n"
+            f"Patterns: {patterns}\n"
             f"Plates:   {tmp_plates}\n"
             f"Channels: {tmp_wavelength_ids}\n"
         )
 
         # Check that only one plate is found
         if len(tmp_plates) > 1:
-            raise Exception(f"{info}ERROR: {len(tmp_plates)} plates detected")
+            raise ValueError(f"{info}ERROR: {len(tmp_plates)} plates detected")
         elif len(tmp_plates) == 0:
-            raise Exception(f"{info}ERROR: No plates detected")
+            raise ValueError(f"{info}ERROR: No plates detected")
         plate = tmp_plates[0]
 
         # If plate already exists in other folder, add suffix
@@ -166,7 +184,7 @@ def create_ome_zarr(
                 )
 
         # Update dict_plate_paths
-        dict_plate_paths[plate] = in_path.parent
+        dict_plate_paths[plate] = in_path
 
     # Check that all channels are in the allowed_channels
     allowed_wavelength_ids = [
@@ -194,31 +212,36 @@ def create_ome_zarr(
         zarrurl = f"{plate}.zarr"
         in_path = dict_plate_paths[plate]
         logger.info(f"Creating {zarrurl}")
-        group_plate = zarr.group(output_path.parent / zarrurl)
+        group_plate = zarr.group(Path(output_path) / zarrurl)
         zarrurls["plate"].append(zarrurl)
 
         # Obtain FOV-metadata dataframe
-        try:
-            # FIXME
-            # Find a smart way to include these metadata files in the dataset
-            # e.g., as resources
-            if metadata_table == "mrf_mlf":
-                mrf_path = f"{in_path}/MeasurementDetail.mrf"
-                mlf_path = f"{in_path}/MeasurementData.mlf"
-                site_metadata, total_files = parse_yokogawa_metadata(
-                    mrf_path, mlf_path
-                )
-                has_mrf_mlf_metadata = True
 
-                # Extract pixel sizes and bit_depth
-                pixel_size_z = site_metadata["pixel_size_z"][0]
-                pixel_size_y = site_metadata["pixel_size_y"][0]
-                pixel_size_x = site_metadata["pixel_size_x"][0]
-                bit_depth = site_metadata["bit_depth"][0]
-        except FileNotFoundError:
-            logger.info("Missing metadata files")
-            has_mrf_mlf_metadata = False
-            pixel_size_x = pixel_size_y = pixel_size_z = 1
+        if metadata_table == "mrf_mlf":
+            mrf_path = f"{in_path}/MeasurementDetail.mrf"
+            mlf_path = f"{in_path}/MeasurementData.mlf"
+
+            site_metadata, number_images_mlf = parse_yokogawa_metadata(
+                mrf_path,
+                mlf_path,
+                filename_patterns=image_glob_patterns,
+            )
+            site_metadata = remove_FOV_overlaps(site_metadata)
+
+        # If a metadata table was passed, load it and use it directly
+        elif metadata_table.endswith(".csv"):
+            logger.warning(
+                "Since a custom metadata table was provided, there will "
+                "be no additional check on the number of image files."
+            )
+            site_metadata = pd.read_csv(metadata_table)
+            site_metadata.set_index(["well_id", "FieldIndex"], inplace=True)
+
+        # Extract pixel sizes and bit_depth
+        pixel_size_z = site_metadata["pixel_size_z"][0]
+        pixel_size_y = site_metadata["pixel_size_y"][0]
+        pixel_size_x = site_metadata["pixel_size_x"][0]
+        bit_depth = site_metadata["bit_depth"][0]
 
         if min(pixel_size_z, pixel_size_y, pixel_size_x) < 1e-9:
             raise Exception(pixel_size_z, pixel_size_y, pixel_size_x)
@@ -226,21 +249,43 @@ def create_ome_zarr(
         # Identify all wells
         plate_prefix = dict_plate_prefixes[plate]
 
-        plate_image_iter = glob(f"{in_path}/{plate_prefix}_{ext_glob_pattern}")
+        patterns = [f"{plate_prefix}_*.{image_extension}"]
+        if image_glob_patterns:
+            patterns.extend(image_glob_patterns)
+        plate_images = glob_with_multiple_patterns(
+            folder=str(in_path), patterns=patterns
+        )
 
         wells = [
-            parse_filename(os.path.basename(fn))["well"]
-            for fn in plate_image_iter
+            parse_filename(os.path.basename(fn))["well"] for fn in plate_images
         ]
         wells = sorted(list(set(wells)))
 
         # Verify that all wells have all channels
         for well in wells:
-            well_image_iter = glob(
-                f"{in_path}/{plate_prefix}_{well}{ext_glob_pattern}"
+            patterns = [f"{plate_prefix}_{well}_*.{image_extension}"]
+            if image_glob_patterns:
+                patterns.extend(image_glob_patterns)
+            well_images = glob_with_multiple_patterns(
+                folder=str(in_path), patterns=patterns
             )
+
+            # Check number of images matches with expected one
+            if metadata_table == "mrf_mlf":
+                num_images_glob = len(well_images)
+                num_images_expected = number_images_mlf[well]
+                if num_images_glob != num_images_expected:
+                    raise ValueError(
+                        f"Wrong number of images for {well=}\n"
+                        f"Expected {num_images_expected} (from mlf file)\n"
+                        f"Found {num_images_glob} files\n"
+                        "Other parameters:\n"
+                        f"  {image_extension=}\n"
+                        f"  {image_glob_patterns=}"
+                    )
+
             well_wavelength_ids = []
-            for fpath in well_image_iter:
+            for fpath in well_images:
                 try:
                     filename_metadata = parse_filename(os.path.basename(fpath))
                     well_wavelength_ids.append(
@@ -347,23 +392,19 @@ def create_ome_zarr(
                 ),
             }
 
-            # FIXME
-            if has_mrf_mlf_metadata:
-                group_tables = group_image.create_group(
-                    "tables/"
-                )  # noqa: F841
+            # Create tables zarr group for ROI tables
+            group_tables = group_image.create_group("tables/")  # noqa: F841
+            well_id = row + column
 
-                # Prepare FOV/well tables
-                FOV_ROIs_table = prepare_FOV_ROI_table(
-                    site_metadata.loc[f"{row+column}"],
-                )
-                # Prepare and write anndata table of well ROIs
-                well_ROIs_table = prepare_well_ROI_table(
-                    site_metadata.loc[f"{row+column}"],
-                )
-                # Write tables
-                write_elem(group_tables, "FOV_ROI_table", FOV_ROIs_table)
-                write_elem(group_tables, "well_ROI_table", well_ROIs_table)
+            # Prepare AnnData tables for FOV/well ROIs
+            FOV_ROIs_table = prepare_FOV_ROI_table(site_metadata.loc[well_id])
+            well_ROIs_table = prepare_well_ROI_table(
+                site_metadata.loc[well_id]
+            )
+
+            # Write AnnData tables in the tables zarr group
+            write_elem(group_tables, "FOV_ROI_table", FOV_ROIs_table)
+            write_elem(group_tables, "well_ROI_table", well_ROIs_table)
 
     # Check that the different images in each well have unique channel labels.
     # Since we currently merge all fields of view in the same image, this check
@@ -371,7 +412,7 @@ def create_ome_zarr(
     # back to one-image-per-field-of-view mode
     for well_path in zarrurls["well"]:
         check_well_channel_labels(
-            well_zarr_path=str(output_path.parent / well_path)
+            well_zarr_path=str(Path(output_path) / well_path)
         )
 
     metadata_update = dict(
@@ -380,7 +421,9 @@ def create_ome_zarr(
         image=zarrurls["image"],
         num_levels=num_levels,
         coarsening_xy=coarsening_xy,
-        original_paths=[str(p) for p in input_paths],
+        image_extension=image_extension,
+        image_glob_patterns=image_glob_patterns,
+        original_paths=input_paths[:],
     )
     return metadata_update
 
@@ -390,13 +433,15 @@ if __name__ == "__main__":
     from fractal_tasks_core._utils import run_fractal_task
 
     class TaskArguments(BaseModel):
-        input_paths: Sequence[Path]
-        output_path: Path
+        input_paths: Sequence[str]
+        output_path: str
         metadata: Dict[str, Any]
+        image_extension: str
+        image_glob_patterns: Optional[list[str]]
         allowed_channels: Sequence[Dict[str, Any]]
-        num_levels: int = 2
-        coarsening_xy: int = 2
-        metadata_table: str = "mrf_mlf"
+        num_levels: Optional[int]
+        coarsening_xy: Optional[int]
+        metadata_table: Optional[str]
 
     run_fractal_task(
         task_function=create_ome_zarr,
