@@ -14,12 +14,12 @@ Copyright 2022 (C)
 Create OME-NGFF zarr group, for multiplexing dataset
 """
 import os
-from glob import glob
 from pathlib import Path
 from typing import Any
 from typing import Dict
 from typing import List
 from typing import Literal
+from typing import Optional
 from typing import Sequence
 from typing import Union
 
@@ -31,6 +31,7 @@ import fractal_tasks_core
 from fractal_tasks_core.lib_channels import check_well_channel_labels
 from fractal_tasks_core.lib_channels import define_omero_channels
 from fractal_tasks_core.lib_channels import validate_allowed_channel_input
+from fractal_tasks_core.lib_glob import glob_with_multiple_patterns
 from fractal_tasks_core.lib_metadata_parsing import parse_yokogawa_metadata
 from fractal_tasks_core.lib_parse_filename_metadata import parse_filename
 from fractal_tasks_core.lib_regions_of_interest import prepare_FOV_ROI_table
@@ -47,9 +48,11 @@ logger = logging.getLogger(__name__)
 
 def create_ome_zarr_multiplex(
     *,
-    input_paths: Sequence[Path],
-    output_path: Path,
+    input_paths: Sequence[str],
+    output_path: str,
     metadata: Dict[str, Any],
+    image_extension: str = "tif",
+    image_glob_patterns: Optional[list[str]] = None,
     allowed_channels: Dict[str, Sequence[Dict[str, Any]]],
     num_levels: int = 2,
     coarsening_xy: int = 2,
@@ -65,11 +68,17 @@ def create_ome_zarr_multiplex(
     Each input_paths should be treated as a different acquisition
 
     :param input_paths: list of image folders for different acquisition
-                        cycles, e.g. in the form ``["/path/cycle1/*.png",
-                        "/path/cycle2/*.png"]``
+                        cycles, e.g. in the form `["/path/cycle1/",
+                        "/path/cycle2/"]`
     :param output_path: parent folder for the output path, e.g.
-                        ``"/outputpath/*.zarr"``
+                        `"/outputpath/"`
     :param metadata: standard fractal argument, not used in this task
+    :param image_extension: Filename extension of images (e.g. `tif` or `png`)
+    :param image_glob_patterns: If specified, only parse images with filenames
+                            that match with all these patterns. Patterns
+                            must be defined as in
+                            https://docs.python.org/3/library/fnmatch.html,
+                            e.g. `image_glob_pattern=["*_B03_*"]`.
     :param allowed_channels: TBD
     :param num_levels: number of resolution-pyramid levels
     :param coarsening_xy: Linear coarsening factor between subsequent levels
@@ -121,10 +130,9 @@ def create_ome_zarr_multiplex(
     # Identify all plates and all channels, per input folders
     dict_acquisitions: Dict = {}
 
-    ext_glob_pattern = input_paths[0].name
-
-    for ind_in_path, in_path in enumerate(input_paths):
+    for ind_in_path, in_path_str in enumerate(input_paths):
         acquisition = str(ind_in_path)
+        in_path = Path(in_path_str)
         dict_acquisitions[acquisition] = {}
 
         actual_wavelength_ids = []
@@ -132,10 +140,16 @@ def create_ome_zarr_multiplex(
         plate_prefixes = []
 
         # Loop over all images
-        input_filename_iter = in_path.parent.glob(ext_glob_pattern)
-        for fn in input_filename_iter:
+        patterns = [f"*.{image_extension}"]
+        if image_glob_patterns:
+            patterns.extend(image_glob_patterns)
+        input_filenames = glob_with_multiple_patterns(
+            folder=in_path_str,
+            patterns=patterns,
+        )
+        for fn in input_filenames:
             try:
-                filename_metadata = parse_filename(fn.name)
+                filename_metadata = parse_filename(Path(fn).name)
                 plate = filename_metadata["plate"]
                 plates.append(plate)
                 plate_prefix = filename_metadata["plate_prefix"]
@@ -145,13 +159,14 @@ def create_ome_zarr_multiplex(
                 actual_wavelength_ids.append(f"A{A}_C{C}")
             except ValueError as e:
                 logger.warning(
-                    f'Skipping "{fn.name}". Original error: ' + str(e)
+                    f'Skipping "{Path(fn).name}". Original error: ' + str(e)
                 )
         plates = sorted(list(set(plates)))
         actual_wavelength_ids = sorted(list(set(actual_wavelength_ids)))
 
         info = (
-            f"Listing all plates/channels from {in_path.as_posix()}\n"
+            "Listing all plates/channels:\n"
+            f"Patterns: {patterns}\n"
             f"Plates:   {plates}\n"
             f"Actual wavelength IDs: {actual_wavelength_ids}\n"
         )
@@ -199,8 +214,8 @@ def create_ome_zarr_multiplex(
         dict_acquisitions[acquisition]["plate"] = plate
         dict_acquisitions[acquisition]["original_plate"] = original_plate
         dict_acquisitions[acquisition]["plate_prefix"] = plate_prefix
-        dict_acquisitions[acquisition]["image_folder"] = str(in_path.parent)
-        dict_acquisitions[acquisition]["original_paths"] = [str(in_path)]
+        dict_acquisitions[acquisition]["image_folder"] = in_path
+        dict_acquisitions[acquisition]["original_paths"] = [in_path]
         dict_acquisitions[acquisition]["actual_channels"] = actual_channels
 
     acquisitions = sorted(list(dict_acquisitions.keys()))
@@ -210,7 +225,7 @@ def create_ome_zarr_multiplex(
     plate = current_plates[0]
 
     zarrurl = dict_acquisitions[acquisitions[0]]["plate"] + ".zarr"
-    full_zarrurl = str(output_path.parent / zarrurl)
+    full_zarrurl = str(Path(output_path) / zarrurl)
     logger.info(f"Creating {full_zarrurl=}")
     group_plate = zarr.group(full_zarrurl)
     group_plate.attrs["plate"] = {
@@ -240,7 +255,7 @@ def create_ome_zarr_multiplex(
             mrf_path = f"{image_folder}/MeasurementDetail.mrf"
             mlf_path = f"{image_folder}/MeasurementData.mlf"
             site_metadata, total_files = parse_yokogawa_metadata(
-                mrf_path, mlf_path
+                mrf_path, mlf_path, filename_patterns=image_glob_patterns
             )
             site_metadata = remove_FOV_overlaps(site_metadata)
 
@@ -259,13 +274,16 @@ def create_ome_zarr_multiplex(
 
         # Identify all wells
         plate_prefix = dict_acquisitions[acquisition]["plate_prefix"]
-        glob_string = f"{image_folder}/{plate_prefix}_{ext_glob_pattern}"
-        logger.info(f"{glob_string=}")
-        plate_image_iter = glob(glob_string)
+        patterns = [f"{plate_prefix}_*.{image_extension}"]
+        if image_glob_patterns:
+            patterns.extend(image_glob_patterns)
+        plate_images = glob_with_multiple_patterns(
+            folder=str(image_folder),
+            patterns=patterns,
+        )
 
         wells = [
-            parse_filename(os.path.basename(fn))["well"]
-            for fn in plate_image_iter
+            parse_filename(os.path.basename(fn))["well"] for fn in plate_images
         ]
         wells = sorted(list(set(wells)))
         logger.info(f"{wells=}")
@@ -273,11 +291,16 @@ def create_ome_zarr_multiplex(
         # Verify that all wells have all channels
         actual_channels = dict_acquisitions[acquisition]["actual_channels"]
         for well in wells:
-            well_image_iter = glob(
-                f"{image_folder}/{plate_prefix}_{well}{ext_glob_pattern}"
+            patterns = [f"{plate_prefix}_{well}_*.{image_extension}"]
+            if image_glob_patterns:
+                patterns.extend(image_glob_patterns)
+            well_images = glob_with_multiple_patterns(
+                folder=str(image_folder),
+                patterns=patterns,
             )
+
             well_wavelength_ids = []
-            for fpath in well_image_iter:
+            for fpath in well_images:
                 try:
                     filename_metadata = parse_filename(os.path.basename(fpath))
                     A = filename_metadata["A"]
@@ -429,7 +452,7 @@ def create_ome_zarr_multiplex(
     # have unique labels
     for well_path in zarrurls["well"]:
         check_well_channel_labels(
-            well_zarr_path=str(output_path.parent / well_path)
+            well_zarr_path=str(Path(output_path) / well_path)
         )
 
     original_paths = {
@@ -444,6 +467,8 @@ def create_ome_zarr_multiplex(
         num_levels=num_levels,
         coarsening_xy=coarsening_xy,
         original_paths=original_paths,
+        image_extension=image_extension,
+        image_glob_patterns=image_glob_patterns,
     )
     return metadata_update
 
@@ -453,13 +478,15 @@ if __name__ == "__main__":
     from fractal_tasks_core._utils import run_fractal_task
 
     class TaskArguments(BaseModel):
-        input_paths: Sequence[Path]
-        output_path: Path
+        input_paths: Sequence[str]
+        output_path: str
         metadata: Dict[str, Any]
+        image_extension: str
+        image_glob_pattern: Optional[list[str]]
         allowed_channels: Dict[str, Sequence[Dict[str, Any]]]
-        num_levels: int = 2
-        coarsening_xy: int = 2
-        metadata_table: Union[Literal["mrf_mlf"], Dict[str, str]] = "mrf_mlf"
+        num_levels: Optional[int]
+        coarsening_xy: Optional[int]
+        metadata_table: Optional[Union[Literal["mrf_mlf"], Dict[str, str]]]
 
     run_fractal_task(
         task_function=create_ome_zarr_multiplex,
