@@ -93,22 +93,22 @@ def correct(
 
 def illumination_correction(
     *,
-    input_paths: Sequence[Path],
-    output_path: Path,
+    input_paths: Sequence[str],
+    output_path: str,
     component: str,
     metadata: Dict[str, Any],
     overwrite: bool = False,
-    new_component: str = None,
+    new_component: Optional[str] = None,
     dict_corr: dict,
     background: int = 100,
 ) -> Dict[str, Any]:
 
     """
-    FIXME
+    FIXME docstring
 
     Example inputs:
-    input_paths: [PosixPath('some_path/*.zarr')]
-    output_path: PosixPath('same_or_other_path/*.zarr')
+    input_paths: ["/some/path"]
+    output_path: "/same/or/other/path/"
     component: myplate.zarr/B/03/0/
     new_component: myplate_new_name.zarr/B/03/0/
     metadata: {...}
@@ -139,15 +139,15 @@ def illumination_correction(
 
     # Defione old/new zarrurls
     plate, well = component.split(".zarr/")
-    in_path = input_paths[0]
-    zarrurl_old = (in_path.parent / component).as_posix()
+    in_path = Path(input_paths[0])
+    zarrurl_old = (in_path / component).as_posix()
     if overwrite:
         zarrurl_new = zarrurl_old
     else:
         new_plate, new_well = new_component.split(".zarr/")
         if new_well != well:
             raise Exception(f"{well=}, {new_well=}")
-        zarrurl_new = (output_path.parent / new_component).as_posix()
+        zarrurl_new = (Path(output_path) / new_component).as_posix()
 
     t_start = time.perf_counter()
     logger.info("Start illumination_correction")
@@ -277,15 +277,15 @@ if __name__ == "__main__":
 
     class TaskArguments(BaseModel):
         # Fractal arguments
-        input_paths: Sequence[Path]
-        output_path: Path
+        input_paths: Sequence[str]
+        output_path: str
         component: str
         metadata: Dict[str, Any]
         # Task-specific arguments
-        overwrite: bool = False
-        new_component: Optional[str] = None
+        overwrite: Optional[bool]
+        new_component: Optional[str]
         dict_corr: dict
-        background: int = 100
+        background: Optional[int]
 
     run_fractal_task(
         task_function=illumination_correction,
