@@ -16,10 +16,10 @@ Task that writes image data to an existing OME-NGFF zarr array
 """
 import logging
 import os
-from glob import glob
 from pathlib import Path
 from typing import Any
 from typing import Dict
+from typing import Optional
 from typing import Sequence
 
 import dask.array as da
@@ -28,6 +28,7 @@ from anndata import read_zarr
 from dask.array.image import imread
 
 from fractal_tasks_core.lib_channels import get_omero_channel_list
+from fractal_tasks_core.lib_glob import glob_with_multiple_patterns
 from fractal_tasks_core.lib_parse_filename_metadata import parse_filename
 from fractal_tasks_core.lib_pyramid_creation import build_pyramid
 from fractal_tasks_core.lib_read_fractal_metadata import (
@@ -58,8 +59,8 @@ def sort_fun(filename: str):
 
 def yokogawa_to_ome_zarr(
     *,
-    input_paths: Sequence[Path],
-    output_path: Path,
+    input_paths: Sequence[str],
+    output_path: str,
     component: str,
     metadata: Dict[str, Any],
     delete_input: bool = False,
@@ -68,8 +69,8 @@ def yokogawa_to_ome_zarr(
     Convert Yokogawa output (png, tif) to zarr file
 
     Example arguments:
-      input_paths[0] = /tmp/output/*.zarr  (Path)
-      output_path = /tmp/output/*.zarr      (Path)
+      input_paths[0] = "/tmp/output/"
+      output_path = "/tmp/output/"
       metadata = {"num_levels": ..., }
       component = plate.zarr/B/03/0/
 
@@ -83,22 +84,29 @@ def yokogawa_to_ome_zarr(
     # Preliminary checks
     if len(input_paths) > 1:
         raise NotImplementedError
-    zarrurl = input_paths[0].parent.as_posix() + f"/{component}"
+    zarrurl = Path(input_paths[0]).as_posix() + f"/{component}"
 
     parameters = get_parameters_from_metadata(
-        keys=["original_paths", "num_levels", "coarsening_xy"],
+        keys=[
+            "original_paths",
+            "num_levels",
+            "coarsening_xy",
+            "image_extension",
+            "image_glob_patterns",
+        ],
         metadata=metadata,
-        image_zarr_path=(output_path.parent / component),
+        image_zarr_path=(Path(output_path) / component),
     )
     original_path_list = parameters["original_paths"]
     num_levels = parameters["num_levels"]
     coarsening_xy = parameters["coarsening_xy"]
+    image_extension = parameters["image_extension"]
+    image_glob_patterns = parameters["image_glob_patterns"]
 
     channels = get_omero_channel_list(image_zarr_path=zarrurl)
     wavelength_ids = [c["wavelength_id"] for c in channels]
 
-    in_path = Path(original_path_list[0]).parent
-    ext = Path(original_path_list[0]).name
+    in_path = Path(original_path_list[0])
 
     # Define well
     component_split = component.split("/")
@@ -126,7 +134,14 @@ def yokogawa_to_ome_zarr(
     max_x = well_indices[0][5]
 
     # Load a single image, to retrieve useful information
-    sample = imread(glob(f"{in_path}/*_{well_ID}_*{ext}")[0])
+    patterns = [f"*_{well_ID}_*.{image_extension}"]
+    if image_glob_patterns:
+        patterns.extend(image_glob_patterns)
+    tmp_images = glob_with_multiple_patterns(
+        folder=str(in_path),
+        patterns=patterns,
+    )
+    sample = imread(tmp_images.pop())
 
     # Initialize zarr
     chunksize = (1, 1, sample.shape[1], sample.shape[2])
@@ -143,17 +158,22 @@ def yokogawa_to_ome_zarr(
     for i_c, wavelength_id in enumerate(wavelength_ids):
         A, C = wavelength_id.split("_")
 
-        glob_path = f"{in_path}/*_{well_ID}_*{A}*{C}{ext}"
-        logger.info(f"glob path: {glob_path}")
-        filenames = sorted(glob(glob_path), key=sort_fun)
+        patterns = [f"*_{well_ID}_*{A}*{C}*.{image_extension}"]
+        if image_glob_patterns:
+            patterns.extend(image_glob_patterns)
+        filenames = glob_with_multiple_patterns(
+            folder=str(in_path),
+            patterns=patterns,
+        )
+        filenames = sorted(list(filenames), key=sort_fun)
         if len(filenames) == 0:
             raise Exception(
                 "Error in yokogawa_to_ome_zarr: len(filenames)=0.\n"
                 f"  in_path: {in_path}\n"
-                f"  ext: {ext}\n"
+                f"  image_extension: {image_extension}\n"
                 f"  well_ID: {well_ID}\n"
                 f"  wavelength_id: {wavelength_id},\n"
-                f"  glob_path: {glob_path}"
+                f"  patterns: {patterns}"
             )
         # Loop over 3D FOV ROIs
         for indices in fov_indices:
@@ -201,11 +221,11 @@ if __name__ == "__main__":
     from fractal_tasks_core._utils import run_fractal_task
 
     class TaskArguments(BaseModel):
-        input_paths: Sequence[Path]
-        output_path: Path
+        input_paths: Sequence[str]
+        output_path: str
         metadata: Dict[str, Any]
         component: str
-        delete_input: bool = False
+        delete_input: Optional[bool]
 
     run_fractal_task(
         task_function=yokogawa_to_ome_zarr,
